// src/utils/getRNStylesFromShikiStyle.ts

import type { TextStyle } from 'react-native'
import type { ThemeRegistration } from 'shiki'

export interface HighlighterStyleSheet {
  [key: string]: TextStyle
}

function normalizeColor(color: string | undefined): string {
  if (!color)
    return 'transparent'
  return color
}

export function getRNStylesFromShikiStyle(theme: ThemeRegistration): HighlighterStyleSheet {
  if (!theme)
    return {}

  try {
    const styles: HighlighterStyleSheet = {
      editor: {
        backgroundColor: normalizeColor(theme.colors?.['editor.background']),
      },
      base: {
        color: normalizeColor(theme.colors?.['editor.foreground']),
      },
    }

    if (theme.tokenColors) {
      theme.tokenColors.forEach((token) => {
        if (!token.settings)
          return

        const style: TextStyle = {}
        if (token.settings.foreground) {
          style.color = token.settings.foreground
        }
        if (token.settings.background) {
          style.backgroundColor = token.settings.background
        }
        if (token.settings.fontStyle) {
          style.fontStyle = token.settings.fontStyle.includes('italic') ? 'italic' : 'normal'
          style.fontWeight = token.settings.fontStyle.includes('bold') ? 'bold' : 'normal'
        }

        const scopes = Array.isArray(token.scope) ? token.scope : [token.scope]
        scopes.forEach((scope) => {
          if (scope) {
            styles[scope] = style
          }
        })
      })
    }

    return styles
  }
  catch (error) {
    return {
      editor: { backgroundColor: '#1e1e1e' },
      base: { color: '#d4d4d4' },
    }
  }
}
